mfACTIVEFIRE <- function(dir.in, dir.out, download=TRUE, collection=5, ylim=NULL, hlim=NULL, vlim=NULL){

	if (length(grep("HDF4", rgdal::getGDALDriverNames()$name))==0) stop("Sorry. rgdal needs to be built with HDF4 support. See ?mfACTIVEFIRE")
	if (!dir.exists(dir.in)) stop("dir.in does not exist")
	if (!dir.exists(dir.out)) stop("dir.out does not exist")

	if (length(ylim)==2){
		if (ylim[1] < 2000){
			stop("MODIS was launched in 2000. ylim[1] must be >= 2000.\n")}
		if (ylim[2] > as.integer(format(Sys.time(), "%Y"))){
			stop(sprintf("Future ylim; ylim[2] must be <= %s", format(Sys.time(), "%Y.")))}
		if (ylim[1] > ylim[2]) stop("ylim[2] must be >= ylim[1]")
		ystart <- ylim[1]; yend <- ylim[2]
	} else {
		if (is.null(ylim)){
			ystart <- NULL; yend <- NULL
		} else stop("ylim must be a vector of length 2 or NULL")
	}
	
	if (length(hlim)==2){
		if (hlim[1] < 0){
			stop("hlim[1] must be >= 0")}
		if (hlim[2] > 35){
			stop("hlim[2] must be <= 35")}
		if (hlim[1] > hlim[2]) stop("hlim[2] must be >= hlim[1]")
		hstart <- hlim[1]; hend <- hlim[2]
	} else {
		if (is.null(hlim)){
			hstart <- NULL; hend <- NULL
		} else stop("hlim must be a vector of length 2 or NULL")
	}

	if (length(vlim)==2){
		if (vlim[1] < 0){
			stop("vlim[1] must be >= 0")}
		if (vlim[2] > 17){
			stop("vlim[2] must be <= 17")}
		if (vlim[1] > vlim[2]) stop("vlim[2] must be >= vlim[1]")
		vstart <- vlim[1]; vend <- vlim[2]
	} else {
		if (is.null(vlim)){
			vstart <- NULL; vend <- NULL
		} else stop("vlim must be a vector of length 2 or NULL")
	}

	if (download){
		cat("starting to download TERRA data\n"); flush.console()
		mfDOWNLOAD(url=sprintf("http://e4ftl01.cr.usgs.gov/MOLT/MOD14A1.%03d/",
				collection), archive=dir.in,
				t0=if (is.null(ystart)) NULL else sprintf("%04d0101", ystart),
				t1=if (is.null(yend)) NULL else sprintf("%04d1231", yend),
				 h0=hstart, h1=hend, v0=vstart, v1=vend)
		cat("starting to download AQUA data\n"); flush.console()
		mfDOWNLOAD(url=sprintf("http://e4ftl01.cr.usgs.gov/MOLA/MYD14A1.%03d/",
				collection), archive=dir.in,
				t0=if (is.null(ystart)) NULL else sprintf("%04d0101", ystart),
				t1=if (is.null(yend)) NULL else sprintf("%04d1231", yend),
				h0=hstart, h1=hend, v0=vstart, v1=vend)
	}

	
	fhdf <- dir(path=dir.in, pattern="^M[OY]D14A1.*\\.hdf$", full.names=TRUE)
	if (length(fhdf)<1) stop("No MOD14A1/MOD14A1 file in dir.in")

	ny <- 1200
	nx <- 1200
	nc <- ny*nx

	h <- as.integer(gsub(".*\\.h([0-9]{2})v[0-9]{2}\\..*", "\\1", basename(fhdf)))
	v <- as.integer(gsub(".*\\.h[0-9]{2}v([0-9]{2})\\..*", "\\1", basename(fhdf)))
	y <- as.integer(gsub(".*\\.A([0-9]{4})[0-9]{3}\\.h.*", "\\1", basename(fhdf)))
	d <- as.integer(gsub(".*\\.A[0-9]{4}([0-9]{3})\\.h.*", "\\1", basename(fhdf)))
	m <- as.integer(format(strptime(
		gsub(".*\\.A([0-9]{4}[0-9]{3})\\.h.*", "\\1", basename(fhdf)), "%Y%j"), "%m"))

	if (is.null(hstart) | is.null(hend)){
		if (!is.null(hstart) | !is.null(hend)) stop("hstart/hend must be given or must be NULL (both of them)")
		hstart <- min(h)
		hend   <- max(h)
	}
	if (is.null(vstart) | is.null(vend)){
		if (!is.null(vstart) | !is.null(vend)) stop("vstart/vend must be given or must be NULL (both of them)")
		vstart <- min(v)
		vend   <- max(v)
	}
	if (is.null(ystart) | is.null(yend)){
		if (!is.null(ystart) | !is.null(yend)) stop("ystart/yend must be given or must be NULL (both of them)")
		ystart <- min(y)
		yend   <- max(y)
	}

	for (H in hstart:hend){
	for (V in vstart:vend){

		pos <- h==H & v==V & y >= ystart & y <= yend
		if (sum(pos) < 1){
			cat(sprintf("no data for tile h%02dv%02d. skip.\n", H, V))
			flush.console()
			next
		}

		# open output files etc.
		fact1  <- file.path(dir.out, sprintf("ACTIFIRE-h%02dv%02d-02-2_1M.dat", H, V)); cact1  <- file(fact1, "wb")
		fact2  <- file.path(dir.out, sprintf("ACTIFIRE-h%02dv%02d-03-2_1Y.dat", H, V)); cact2  <- file(fact2, "wb")
		fact3  <- file.path(dir.out, sprintf("ACTIFIRE-h%02dv%02d-03-3_1Y.dat", H, V)); cact3  <- file(fact3, "wb")
		fact4  <- file.path(dir.out, sprintf("ACTIFIRE-h%02dv%02d-04-2_OA.dat", H, V)); cact4  <- file(fact4, "wb")
		fact5  <- file.path(dir.out, sprintf("ACTIFIRE-h%02dv%02d-04-3_OA.dat", H, V)); cact5  <- file(fact5, "wb")
		fact6  <- file.path(dir.out, sprintf("ACTIFIRE-h%02dv%02d-04-4_OA.dat", H, V)); cact6  <- file(fact6, "wb")
		fact7  <- file.path(dir.out, sprintf("ACTIFIRE-h%02dv%02d-05-2_1M.dat", H, V)); cact7  <- file(fact7, "wb")
		fact8  <- file.path(dir.out, sprintf("ACTIFIRE-h%02dv%02d-03-T_1Y.dat", H, V)); cact8  <- file(fact8, "wb")
		ferr1  <- file.path(dir.out, sprintf("ACTIQUAL-h%02dv%02d-02-0_1M.dat", H, V)); cerr1  <- file(ferr1, "wb")
		ferr2  <- file.path(dir.out, sprintf("ACTIQUAL-h%02dv%02d-03-0_1Y.dat", H, V)); cerr2  <- file(ferr2, "wb")
		ferr3  <- file.path(dir.out, sprintf("ACTIQUAL-h%02dv%02d-04-0_OA.dat", H, V)); cerr3  <- file(ferr3, "wb")
		ferr4  <- file.path(dir.out, sprintf("ACTIQUAL-h%02dv%02d-05-0_1M.dat", H, V)); cerr4  <- file(ferr4, "wb")
		fint1  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-02-1_1M.dat", H, V)); cint1  <- file(fint1, "wb")
		fint2  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-03-1_1Y.dat", H, V)); cint2  <- file(fint2, "wb")
		fint3  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-03-2_1Y.dat", H, V)); cint3  <- file(fint3, "wb")
		fint4  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-03-3_1Y.dat", H, V)); cint4  <- file(fint4, "wb")
		fint5  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-04-1_OA.dat", H, V)); cint5  <- file(fint5, "wb")
		fint6  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-04-2_OA.dat", H, V)); cint6  <- file(fint6, "wb")
		fint7  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-04-3_OA.dat", H, V)); cint7  <- file(fint7, "wb")
		fint8  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-05-1_OA.dat", H, V)); cint8  <- file(fint8, "wb")
		fint9  <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-05-2_OA.dat", H, V)); cint9  <- file(fint9, "wb")
		fint10 <- file.path(dir.out, sprintf("ACTI-INT-h%02dv%02d-05-3_OA.dat", H, V)); cint10 <- file(fint10,"wb")

		# init overall stats
		om_sumfire <- rep(0L, nc)
		oy_sumfire <- rep(0L, nc)
		o_maxfrp  <- rep(0L, nc)
		o_sumfrp  <- rep(0L, nc)
		o_sumerro <- rep(0L, nc)
		noerro <- 0

		# init agg-month stats
		mm_sumfire <- matrix(0L, 12, nc)
		mm_maxfrp  <- matrix(0L, 12, nc)
		mm_sumfrp  <- matrix(0L, 12, nc)
		mm_sumerro <- matrix(0L, 12, nc)
		nmmerro <- rep(0, 12)

		km <- 0
		ky <- 0; cy <- integer(0)
		dat <- character(0)

		for (Y in ystart:yend){

			pos <- h==H & v==V & y==Y
			if (sum(pos) < 1) next

			# init year stats
			y_sumfire <- rep(0L, nc)
			y_monfire <- rep(0L, nc)
			y_maxfrp  <- rep(0,  nc)
			y_sumfrp  <- rep(0,  nc)
			y_sumerro <- rep(0L, nc)
			nyerro <- 0

			for (M in 1:12){

				pos <- h==H & v==V & y==Y & m==M
				if (sum(pos) < 1) next

				# init month stats
				m_isfire <- rep(0L, nc)
				m_maxfrp <- rep(0,  nc)
				m_sumerro <-rep(0L, nc)

				npos <- sum(pos); pos <- which(pos); np <- 0
				for (i in pos){

					f <- sprintf("HDF4_EOS:EOS_GRID:%s:MODIS_Grid_Daily_Fire:FireMask", fhdf[i])
					r <- rgdal::readGDAL(f, silent=TRUE)

					data <- as.matrix(r@data)
					nodata <- is.na(data)
					if (sum(nodata)>0) data[nodata] <- 0

					erro <- (data > 0 & data < 5) | data == 6
					m_sumerro <- m_sumerro+rowSums(erro)
					fire <- data > 6
					np <- np+dim(data)[2]

					if (sum(fire) == 0){
						next
					} else { 

						f <- sprintf("HDF4_EOS:EOS_GRID:%s:MODIS_Grid_Daily_Fire:MaxFRP", fhdf[i])
						r <- rgdal::readGDAL(f, silent=TRUE)
						data <- as.matrix(r@data)
						nodata <- is.na(data) | !fire
						if (sum(nodata)>0) data[nodata] <- 0

					}

					m_isfire <- m_isfire+rowSums(fire)
					m_maxfrp <- pmax(m_maxfrp,rowSums(data))

				}

				y_sumerro <- y_sumerro+m_sumerro
				m_erro <- m_sumerro/np*100
				storage.mode(m_erro) <- "integer"
				nyerro <- nyerro+np

				mm_sumerro[M,] <- mm_sumerro[M,]+m_sumerro
				nmmerro[M] <- nmmerro[M] + np

				m_isfire <- m_isfire > 0
				y_sumfire <- y_sumfire+m_isfire
				mm_sumfire[M,] <- mm_sumfire[M,]+m_isfire

				y_monfire[m_isfire] <- as.integer(M)

				mm_maxfrp[M,] <- pmax(mm_maxfrp[M,],m_maxfrp)
				mm_sumfrp[M,] <- mm_sumfrp[M,]+m_maxfrp
				y_maxfrp <- pmax(y_maxfrp,m_maxfrp)
				y_sumfrp <- y_sumfrp+m_maxfrp
				storage.mode(m_maxfrp) <- "integer"

				writeBin(m_isfire, cact1, 1)
				writeBin(m_maxfrp, cint1, 4)
				writeBin(m_erro, cerr1, 1)

				dat <- c(dat, sprintf("%04d-%02d", Y, M))
				km <- km+1

				cat(sprintf("\rdone with %04d-%02d", Y, M)); flush.console()

			}

			o_sumerro <- o_sumerro+y_sumerro
			y_erro <- y_sumerro/nyerro*100
			storage.mode(y_erro) <- "integer"
			noerro <- noerro+nyerro

			y_isfire <- y_sumfire > 0
			y_avgfrp <- y_sumfrp/y_sumfire
			y_avgfrp[which(!is.finite(y_avgfrp))] <- 0

			oy_sumfire <- oy_sumfire+y_isfire
			om_sumfire <- om_sumfire+y_sumfire

			o_maxfrp <- pmax(o_maxfrp,y_maxfrp)
			o_sumfrp <- o_sumfrp+y_maxfrp

			storage.mode(y_avgfrp) <- "integer"
			storage.mode(y_maxfrp) <- "integer"
			storage.mode(y_sumfrp) <- "integer"

			writeBin(y_sumfire, cact2, 1)
			writeBin(y_isfire,  cact3, 1)
			writeBin(y_monfire, cact8, 1)

			writeBin(y_maxfrp, cint2, 4)
			writeBin(y_sumfrp, cint3, 4)
			writeBin(y_avgfrp, cint4, 4)

			writeBin(y_erro, cerr2, 1)

			ky <- ky+1
			cy <- c(cy, Y)

		}


		o_erro <- o_sumerro/noerro*100
		storage.mode(o_erro) <- "integer"

		o_isfire <- om_sumfire > 0
		o_avgfrp <- o_sumfrp/om_sumfire
		o_avgfrp[which(!is.finite(o_avgfrp))] <- 0

		storage.mode(o_avgfrp)  <- "integer"
		storage.mode(o_maxfrp)  <- "integer"
		storage.mode(o_sumfrp)  <- "integer"
		storage.mode(mm_maxfrp) <- "integer"
		storage.mode(mm_sumfrp) <- "integer"

		writeBin(om_sumfire, cact4, 1)
		writeBin(oy_sumfire, cact5, 1)
		writeBin(o_isfire,   cact6, 1)
		for (m in 1:12) writeBin(mm_sumfire[m,], cact7, 1)
		writeBin(o_maxfrp, cint5, 4)
		writeBin(o_sumfrp, cint6, 4)
		writeBin(o_avgfrp, cint7, 4)
		for (m in 1:12) writeBin(mm_maxfrp[m,], cint8, 4)
		for (m in 1:12) writeBin(mm_sumfrp[m,], cint9, 4)
		for (m in 1:12){
			mm_avgfrp <- mm_sumfrp[m,]/mm_sumfire[m,]
			mm_avgfrp[which(!is.finite(mm_avgfrp))] <- 0
			storage.mode(mm_avgfrp) <- "integer"
			writeBin(mm_avgfrp, cint10, 4)
		}

		for (m in 1:12){
			mm_erro <- mm_sumerro[m,]/nmmerro[m]*100
			storage.mode(mm_erro) <- "integer"
			writeBin(mm_erro, cerr4, 1)
		}
		writeBin(o_erro, cerr3, 1)

		cat("\n")


		close(cact1)
		mfENVIHDR(fact1, ny, nx, km, 1, "BSQ", "fire?")
		mfBNAMES(fact1, dat, TRUE)

		close(cact2)
		mfENVIHDR(fact2, ny, nx, ky, 1, "BSQ", "# of fire-months")
		mfBNAMES(fact2, cy, TRUE)

		close(cact3)
		mfENVIHDR(fact3, ny, nx, ky, 1, "BSQ", "fire?")
		mfBNAMES(fact3, cy, TRUE)

		close(cact4)
		mfENVIHDR(fact4, ny, nx, 1, 1, "BSQ", "# of fire-months")
		mfBNAMES(fact4, "overall", TRUE)

		close(cact5)
		mfENVIHDR(fact5, ny, nx, 1, 1, "BSQ", "# of fire-years")
		mfBNAMES(fact5, "overall", TRUE)

		close(cact6)
		mfENVIHDR(fact6, ny, nx, 1, 1, "BSQ", "fire?")
		mfBNAMES(fact6, "overall", TRUE)

		close(cact7)
		mfENVIHDR(fact7, ny, nx, 12, 1, "BSQ", "# of fire-years")
		mfBNAMES(fact7, formatC(1:12, width=2, flag=0), TRUE)

		close(cact8)
		mfENVIHDR(fact8, ny, nx, ky, 1, "BSQ", "month of latest fire")
		mfBNAMES(fact8, cy, TRUE)

		close(cerr1)
		mfENVIHDR(ferr1, ny, nx, km, 1, "BSQ", "% error")
		mfBNAMES(ferr1, dat, TRUE)

		close(cerr2)
		mfENVIHDR(ferr2, ny, nx, ky, 1, "BSQ", "% error")
		mfBNAMES(ferr2, cy, TRUE)

		close(cerr3)
		mfENVIHDR(ferr3, ny, nx, 1, 1, "BSQ", "% error")
		mfBNAMES(ferr3, "overall", TRUE)

		close(cerr4)
		mfENVIHDR(ferr4, ny, nx, 12, 1, "BSQ", "% error")
		mfBNAMES(ferr4, formatC(1:12, width=2, flag=0), TRUE)

		close(cint1)
		mfENVIHDR(fint1, ny, nx, km, 3, "BSQ", "max FRP in MW")
		mfBNAMES(fint1, dat, TRUE)

		close(cint2)
		mfENVIHDR(fint2, ny, nx, ky, 3, "BSQ", "max FRP in MW")
		mfBNAMES(fint2, cy, TRUE)

		close(cint3)
		mfENVIHDR(fint3, ny, nx, ky, 3, "BSQ", "sum FRP in MW")
		mfBNAMES(fint3, cy, TRUE)

		close(cint4)
		mfENVIHDR(fint4, ny, nx, ky, 3, "BSQ", "mean FRP in MW")
		mfBNAMES(fint4, cy, TRUE)

		close(cint5)
		mfENVIHDR(fint5, ny, nx, 1, 3, "BSQ", "max FRP in MW")
		mfBNAMES(fint5, "overall", TRUE)

		close(cint6)
		mfENVIHDR(fint6, ny, nx, 1, 3, "BSQ", "sum FRP in MW")
		mfBNAMES(fint6, "overall", TRUE)

		close(cint7)
		mfENVIHDR(fint7, ny, nx, 1, 3, "BSQ", "mean FRP in MW")
		mfBNAMES(fint7, "overall", TRUE)

		close(cint8)
		mfENVIHDR(fint8, ny, nx, 12, 3, "BSQ", "max FRP in MW")
		mfBNAMES(fint8, formatC(1:12, width=2, flag=0), TRUE)

		close(cint9)
		mfENVIHDR(fint9, ny, nx, 12, 3, "BSQ", "sum FRP in MW")
		mfBNAMES(fint9, formatC(1:12, width=2, flag=0), TRUE)

		close(cint10)
		mfENVIHDR(fint10, ny, nx, 12, 3, "BSQ", "mean FRP in MW")
		mfBNAMES(fint10, formatC(1:12, width=2, flag=0), TRUE)

		dfile <- file.path(dir.out, sprintf("DATELIST-h%02dv%02d-02-0_1M.txt", H, V))
		write(dat, dfile, ncolumns=1)
		dfile <- file.path(dir.out, sprintf("DATELIST-h%02dv%02d-03-0_1Y.txt", H, V))
		write(cy, dfile, ncolumns=1)
		dfile <- file.path(dir.out, sprintf("DATELIST-h%02dv%02d-05-0_1M.txt", H, V))
		write(formatC(1:12, width=2, flag=0), dfile, ncolumns=1)
	}
	}

}

