mfBURNEDAREA <- function(dir.in, dir.out, download=TRUE, collection=5, ylim=NULL, hlim=NULL, vlim=NULL){

	if (length(grep("HDF4", rgdal::getGDALDriverNames()$name))==0) stop("Sorry. rgdal needs to be built with HDF4 support. See ?mfBURNEDAREA")
	if (!dir.exists(dir.in)) stop("dir.in does not exist")
	if (!dir.exists(dir.out)) stop("dir.out does not exist")

	if (length(ylim)==2){
		if (ylim[1] < 2000){
			stop("MODIS was launched in 2000. ylim[1] must be >= 2000.\n")}
		if (ylim[2] > as.integer(format(Sys.time(), "%Y"))){
			stop(sprintf("Future ylim; ylim[2] must be <= %s", format(Sys.time(), "%Y.")))}
		if (ylim[1] > ylim[2]) stop("ylim[2] must be >= ylim[1]")
		ystart <- ylim[1]; yend <- ylim[2]
	} else {
		if (is.null(ylim)){
			ystart <- NULL; yend <- NULL
		} else stop("ylim must be a vector of length 2 or NULL")
	}
	
	if (length(hlim)==2){
		if (hlim[1] < 0){
			stop("hlim[1] must be >= 0")}
		if (hlim[2] > 35){
			stop("hlim[2] must be <= 35")}
		if (hlim[1] > hlim[2]) stop("hlim[2] must be >= hlim[1]")
		hstart <- hlim[1]; hend <- hlim[2]
	} else {
		if (is.null(hlim)){
			hstart <- NULL; hend <- NULL
		} else stop("hlim must be a vector of length 2 or NULL")
	}

	if (length(vlim)==2){
		if (vlim[1] < 0){
			stop("vlim[1] must be >= 0")}
		if (vlim[2] > 17){
			stop("vlim[2] must be <= 17")}
		if (vlim[1] > vlim[2]) stop("vlim[2] must be >= vlim[1]")
		vstart <- vlim[1]; vend <- vlim[2]
	} else {
		if (is.null(vlim)){
			vstart <- NULL; vend <- NULL
		} else stop("vlim must be a vector of length 2 or NULL")
	}

	if (download){
		cat("starting to download TERRA/AQUA data\n"); flush.console()
		mfDOWNLOAD(url=sprintf("http://e4ftl01.cr.usgs.gov/MOTA/MCD45A1.%03d/",
				collection), archive=dir.in,
				t0=if (is.null(ystart)) NULL else sprintf("%04d0101", ystart),
				t1=if (is.null(yend)) NULL else sprintf("%04d1231", yend),
				 h0=hstart, h1=hend, v0=vstart, v1=vend)
	}

	fhdf <- dir(path=dir.in, pattern="^MCD45A1.*\\.hdf$", full.names=TRUE)
	if (length(fhdf)<1) stop("No MCD45A1 file in dir.in")

	ny <- 2400
	nx <- 2400
	nc <- ny*nx

	h <- as.integer(gsub(".*\\.h([0-9]{2})v[0-9]{2}\\..*", "\\1", basename(fhdf)))
	v <- as.integer(gsub(".*\\.h[0-9]{2}v([0-9]{2})\\..*", "\\1", basename(fhdf)))
	y <- as.integer(gsub(".*\\.A([0-9]{4})[0-9]{3}\\.h.*", "\\1", basename(fhdf)))
	d <- as.integer(gsub(".*\\.A[0-9]{4}([0-9]{3})\\.h.*", "\\1", basename(fhdf)))
	m <- as.integer(format(strptime(
		gsub(".*\\.A([0-9]{4}[0-9]{3})\\.h.*", "\\1", basename(fhdf)), "%Y%j"), "%m"))

	if (is.null(hstart) | is.null(hend)){
		if (!is.null(hstart) | !is.null(hend)) stop("hstart/hend must be given or must be NULL (both of them)")
		hstart <- min(h)
		hend   <- max(h)
	}
	if (is.null(vstart) | is.null(vend)){
		if (!is.null(vstart) | !is.null(vend)) stop("vstart/vend must be given or must be NULL (both of them)")
		vstart <- min(v)
		vend   <- max(v)
	}
	if (is.null(ystart) | is.null(yend)){
		if (!is.null(ystart) | !is.null(yend)) stop("ystart/yend must be given or must be NULL (both of them)")
		ystart <- min(y)
		yend   <- max(y)
	}

	for (H in hstart:hend){
	for (V in vstart:vend){

		pos <- h==H & v==V & y >= ystart & y <= yend
		if (sum(pos) < 1){
			cat(sprintf("no data for tile h%02dv%02d. skip.\n", H, V))
			flush.console()
			next
		}

		# open output files etc.
		fbrn1  <- file.path(dir.out, sprintf("BURNAREA-h%02dv%02d-02-2_1M.dat", H, V)); cbrn1  <- file(fbrn1, "wb")
		fbrn2  <- file.path(dir.out, sprintf("BURNAREA-h%02dv%02d-03-2_1Y.dat", H, V)); cbrn2  <- file(fbrn2, "wb")
		fbrn3  <- file.path(dir.out, sprintf("BURNAREA-h%02dv%02d-03-3_1Y.dat", H, V)); cbrn3  <- file(fbrn3, "wb")
		fbrn4  <- file.path(dir.out, sprintf("BURNAREA-h%02dv%02d-04-2_OA.dat", H, V)); cbrn4  <- file(fbrn4, "wb")
		fbrn5  <- file.path(dir.out, sprintf("BURNAREA-h%02dv%02d-04-3_OA.dat", H, V)); cbrn5  <- file(fbrn5, "wb")
		fbrn6  <- file.path(dir.out, sprintf("BURNAREA-h%02dv%02d-04-4_OA.dat", H, V)); cbrn6  <- file(fbrn6, "wb")
		fbrn7  <- file.path(dir.out, sprintf("BURNAREA-h%02dv%02d-05-2_1M.dat", H, V)); cbrn7  <- file(fbrn7, "wb")
		fbrn8  <- file.path(dir.out, sprintf("BURNAREA-h%02dv%02d-03-T_1Y.dat", H, V)); cbrn8  <- file(fbrn8, "wb")
		ferr1  <- file.path(dir.out, sprintf("BURNQUAL-h%02dv%02d-02-0_1M.dat", H, V)); cerr1  <- file(ferr1, "wb")
		ferr2  <- file.path(dir.out, sprintf("BURNQUAL-h%02dv%02d-03-0_1Y.dat", H, V)); cerr2  <- file(ferr2, "wb")
		ferr3  <- file.path(dir.out, sprintf("BURNQUAL-h%02dv%02d-04-0_OA.dat", H, V)); cerr3  <- file(ferr3, "wb")
		ferr4  <- file.path(dir.out, sprintf("BURNQUAL-h%02dv%02d-05-0_1M.dat", H, V)); cerr4  <- file(ferr4, "wb")


		# init overall stats
		om_sumfire <- rep(0L, nc)
		oy_sumfire <- rep(0L, nc)
		o_sumerro <- rep(0L, nc)
		noerro <- 0

		# init agg-month stats
		mm_sumfire <- matrix(0L, 12, nc)
		mm_sumerro <- matrix(0L, 12, nc)
		nmmerro <- rep(0, 12)

		old <- rep(FALSE, nc)

		km <- 0
		ky <- 0; cy <- integer(0)
		dat <- character(0)

		for (Y in ystart:yend){

			pos <- h==H & v==V & y==Y
			if (sum(pos) < 1) next

			# init year stats
			y_sumfire <- rep(0L, nc)
			y_monfire <- rep(0L, nc)
			y_sumerro <- rep(0L, nc)
			nyerro <- 0

			for (M in 1:12){

				pos <- h==H & v==V & y==Y & m==M
				if (sum(pos) < 1) next

				# init month stats
				m_isfire <- rep(0L, nc)
				m_sumerro <-rep(0L, nc)

				npos <- sum(pos); pos <- which(pos); np <- 0
				for (i in pos){

					f <- sprintf("HDF4_EOS:EOS_GRID:%s:MOD_GRID_Monthly_500km_BA:burndate", fhdf[i])
					r <- rgdal::readGDAL(f, silent=TRUE)

					data <- r@data[,1]
					nodata <- is.na(data)
					if (sum(nodata)>0) data[nodata] <- 0

					f <- sprintf("HDF4_EOS:EOS_GRID:%s:MOD_GRID_Monthly_500km_BA:ba_qa", fhdf[i])
					r <- rgdal::readGDAL(f, silent=TRUE)

					qual <- r@data[,1]
					nodata <- is.na(qual)
					if (sum(nodata)>0) qual[nodata] <- 0

					erro <- data > 366 | data < 0
					m_sumerro <- m_sumerro+erro
					fire <- (data >= 1 & data <= 366) & (qual == 1 | qual == 2)
					doub <- fire & old
					if (sum(doub)>0) fire[doub] <- FALSE
					np <- np+1

					m_isfire <- m_isfire+fire
					old <- fire

				}

				y_sumerro <- y_sumerro+m_sumerro
				m_erro <- m_sumerro/np*100
				storage.mode(m_erro) <- "integer"
				nyerro <- nyerro+np

				mm_sumerro[M,] <- mm_sumerro[M,]+m_sumerro
				nmmerro[M] <- nmmerro[M] + np

				m_isfire <- m_isfire > 0
				y_sumfire <- y_sumfire+m_isfire
				mm_sumfire[M,] <- mm_sumfire[M,]+m_isfire

				y_monfire[m_isfire] <- as.integer(M)

				writeBin(m_isfire, cbrn1, 1)
				writeBin(m_erro, cerr1, 1)

				dat <- c(dat, sprintf("%04d-%02d", Y, M))
				km <- km+1

				cat(sprintf("\rdone with %04d-%02d", Y, M)); flush.console()

			}

			o_sumerro <- o_sumerro+y_sumerro
			y_erro <- y_sumerro/nyerro*100
			storage.mode(y_erro) <- "integer"
			noerro <- noerro+nyerro

			y_isfire <- y_sumfire > 0

			oy_sumfire <- oy_sumfire+y_isfire
			om_sumfire <- om_sumfire+y_sumfire

			writeBin(y_sumfire, cbrn2, 1)
			writeBin(y_isfire,  cbrn3, 1)
			writeBin(y_monfire, cbrn8, 1)

			writeBin(y_erro, cerr2, 1)

			ky <- ky+1
			cy <- c(cy, Y)

		}


		o_erro <- o_sumerro/noerro*100
		storage.mode(o_erro) <- "integer"

		o_isfire <- om_sumfire > 0

		writeBin(om_sumfire, cbrn4, 1)
		writeBin(oy_sumfire, cbrn5, 1)
		writeBin(o_isfire,   cbrn6, 1)
		for (m in 1:12) writeBin(mm_sumfire[m,], cbrn7, 1)

		for (m in 1:12){
			mm_erro <- mm_sumerro[m,]/nmmerro[m]*100
			storage.mode(mm_erro) <- "integer"
			writeBin(mm_erro, cerr4, 1)
		}
		writeBin(o_erro, cerr3, 1)

		cat("\n")


		close(cbrn1)
		mfENVIHDR(fbrn1, ny, nx, km, 1, "BSQ", "fire?")
		mfBNAMES(fbrn1, dat, TRUE)

		close(cbrn2)
		mfENVIHDR(fbrn2, ny, nx, ky, 1, "BSQ", "# of fire-months")
		mfBNAMES(fbrn2, cy, TRUE)

		close(cbrn3)
		mfENVIHDR(fbrn3, ny, nx, ky, 1, "BSQ", "fire?")
		mfBNAMES(fbrn3, cy, TRUE)

		close(cbrn4)
		mfENVIHDR(fbrn4, ny, nx, 1, 1, "BSQ", "# of fire-months")
		mfBNAMES(fbrn4, "overall", TRUE)

		close(cbrn5)
		mfENVIHDR(fbrn5, ny, nx, 1, 1, "BSQ", "# of fire-years")
		mfBNAMES(fbrn5, "overall", TRUE)

		close(cbrn6)
		mfENVIHDR(fbrn6, ny, nx, 1, 1, "BSQ", "fire?")
		mfBNAMES(fbrn6, "overall", TRUE)

		close(cbrn7)
		mfENVIHDR(fbrn7, ny, nx, 12, 1, "BSQ", "# of fire-years")
		mfBNAMES(fbrn7, formatC(1:12, width=2, flag=0), TRUE)

		close(cbrn8)
		mfENVIHDR(fbrn8, ny, nx, ky, 1, "BSQ", "month of latest fire")
		mfBNAMES(fbrn8, cy, TRUE)

		close(cerr1)
		mfENVIHDR(ferr1, ny, nx, km, 1, "BSQ", "% error")
		mfBNAMES(ferr1, dat, TRUE)

		close(cerr2)
		mfENVIHDR(ferr2, ny, nx, ky, 1, "BSQ", "% error")
		mfBNAMES(ferr2, cy, TRUE)

		close(cerr3)
		mfENVIHDR(ferr3, ny, nx, 1, 1, "BSQ", "% error")
		mfBNAMES(ferr3, "overall", TRUE)

		close(cerr4)
		mfENVIHDR(ferr4, ny, nx, 12, 1, "BSQ", "% error")
		mfBNAMES(ferr4, formatC(1:12, width=2, flag=0), TRUE)

		dfile <- file.path(dir.out, sprintf("DATELIST-h%02dv%02d-02-0_1M.txt", H, V))
		write(dat, dfile, ncolumns=1)
		dfile <- file.path(dir.out, sprintf("DATELIST-h%02dv%02d-03-0_1Y.txt", H, V))
		write(cy, dfile, ncolumns=1)
		dfile <- file.path(dir.out, sprintf("DATELIST-h%02dv%02d-05-0_1M.txt", H, V))
		write(formatC(1:12, width=2, flag=0), dfile, ncolumns=1)
	}
	}

}


