\name{mfBURNEDAREA}
\alias{mfBURNEDAREA}
\title{
Derive Burned Area Products.
}
\description{
This function derives the Burned Area products documented in the scientific publication.\cr
The MODIS images (MCD45A1 product) can be automatically downloaded.\cr
This function can process multiple MODIS tiles in a batch.\cr
The requested period of time can be specified.
}
\usage{
mfBURNEDAREA(dir.in, dir.out, download = TRUE, collection = 5,
             ylim = NULL, hlim = NULL, vlim = NULL)
}
\arguments{
  \item{dir.in}{
Directory for the MODIS standard products.
}
  \item{dir.out}{
Directory for the derived Burned Area products.
}
  \item{download}{
(optional) Download the data? Defaults to TRUE.
}
  \item{collection}{
(optional) MODIS standard product collection number. Defaults to 5.
}
  \item{ylim}{
(optional) The years to be considered [2000,today]. Either NULL or 2-length vector.
}
  \item{hlim}{
(optional) The horizontal tiles to be considered [0,35]. Either NULL or 2-length vector.
}
  \item{vlim}{
(optional) The vertical tiles to be considered [0,17]. Either NULL or 2-length vector.
}
}
\details{
\code{dir.in} must be an existing directory. All required MODIS images (\href{https://lpdaac.usgs.gov/dataset_discovery/modis/modis_products_table/mcd45a1}{MCD45A1 products} in hdf format) must be placed in this directory, subdirectories are not allowed. If \code{download} == TRUE, the MCD14A1 products will be downloaded to this directory automatically; see details to \code{download} below. If \code{download} == FALSE, the images must be acquired prior to running this tool.\cr\cr
\code{dir.out} must be an existing directory, and will be used for the generation of the BUrned Area products described in the scientific publication.\cr\cr
\code{download} enables the automatic download of the requested MODIS products from the HTTP Data Pool of LP DAAC \href{https://lpdaac.usgs.gov/data_access/data_pool}{Data Pool} of the LP DAAC. Existing images won't be downloaded again. Note that the URL, or the access situation may change at any time without notice, in which case the download functionality will be broken. Note that server maintenance is scheduled weekly, which will temporariliy disable the download functionality. Internet access is required.\cr\cr
The method was developed with \code{collection} 5 data. It may be possible to also process data from other collections, though we do not guarantee this.\cr\cr
The requested period of time may be specified with \code{ylim}. Note that MODIS was commissioned in 2000, which is the allowed minimum, the current year is the allowed maximum. If \code{ylim} is not specified, all available data will be considered.\cr\cr
The requested spatial extent may be specified with \code{hlim}, and \code{vlim}. These numbers refer to the \href{http://modis-land.gsfc.nasa.gov/MODLAND_grid.html}{Sinusoidal Tile Grid} used by the MODIS land products. If none of these parameters is specified, all available data will be considered, i.e. global coverage. Note that this will take considerable time.
}
\section{Output}{
1] Fire counts\cr
   BURNAREA-hxxvxx-02-2_1M.dat:  Fire [0/1] per month?\cr
   BURNAREA-hxxvxx-03-2_1Y.dat:  # of fire-months per year\cr
   BURNAREA-hxxvxx-03-3_1Y.dat:  Fire [0/1] per year?\cr
   BURNAREA-hxxvxx-04-2_OA.dat:  # of fire-months in complete time-series\cr
   BURNAREA-hxxvxx-04-3_OA.dat:  # of fire-years  in complete time series\cr
   BURNAREA-hxxvxx-04-4_OA.dat:  Fire [0/1] in complete time-series?\cr
   BURNAREA-hxxvxx-05-2_1M.dat:  # of fire-years per month [Jan-Dec]\cr
   BURNAREA-hxxvxx-03-T_1Y.dat:  Month of latest fire per year\cr\cr
2] Invalid observations\cr
   BURNQUAL-hxxvxx-02-0_1M.dat:  \% of invalid data per month\cr
   BURNQUAL-hxxvxx-03-0_1Y.dat:  \% of invalid data per year\cr
   BURNQUAL-hxxvxx-04-0_OA.dat:  \% of invalid data in complete time-series\cr
   BURNQUAL-hxxvxx-05-0_1M.dat:  \% of invalid data per month [Jan-Dec]\cr\cr
3] Date-Files\cr
   DATELIST-hxxvxx-02-0_1M.txt:  List of all available months\cr
   DATELIST-hxxvxx-03-0_1Y.txt:  List of all available years\cr
   DATELIST-hxxvxx-05-0_1M.txt:  List of all months [Jan-Dec]\cr
}
\references{
Stellmes, M., Frantz, D., Finckh, M., and Revermann, R. (2013). Fire frequency, fire seasonality and fire intensity within the Okavango region derived from MODIS fire products. In Oldeland, J., Erb, C., Finckh, M., and Juergens, N. (Eds.), Special Volume: Environmental Assessments in the Okavango Region, Biodiversity & Ecology 5, 351-362. DOI: 10.7809/b-e.00288\cr\cr
Roy, D.P. , Lewis, P.E. , Justice, C.O. (2002). Burned area mapping using multi-temporal moderate spatial resolution data - a bi-directional reflectance model-based expectation approach. Remote Sensing of Environment 83: 263-286. DOI: 10.1016/S0034-4257(02)00077-9\cr\cr
Roy, D.P. , Jin, Y. , Lewis, P.E. , Justice, C.O. (2005). Prototyping a global algorithm for systematic fire-affected area mapping using MODIS time series data. Remote Sensing of Environment 97: 137-162. DOI: 10.1016/j.rse.2005.04.007
}
\author{
David Frantz
}
\section{Acknowledgements}{
The MCD45A1 product is retrieved from the Data Pool, courtesy of the NASA EOSDIS Land Processes Distributed Active Archive Center (LP DAAC), USGS/Earth Resources Observation and Science (EROS) Center, Sioux Falls, South Dakota, https://lpdaac.usgs.gov/data_access/
data_pool.
}

\section{Warning}{
It has to be noted that this function may not be working on a standard Windows installation. MODIS data are distributed in the Hierarchical Data Format, version 4. To our knowledge, there exists no R package with native HDF4 support in Windows. This function relies on the rgdal package for reading the MODIS data, and rgdal must be built with HDF4 support. This is standard in Linux builds, but not in Windows. Note that we cannot provide support for building rgdal. We encourage the user to use a Linux system.
:}
\seealso{
\code{\link{mfACTIVEFIRE}}.
}
\examples{
\dontrun{
# Download and Process the full global time series.
# Not adviced, though.
mydir1 <- "path-for-download"
mydir2 <- "path-for-products"
mfBURNEDAREA(mydir1, mydir2)

# Download and Process the full Southern African time series.
mfBURNEDAREA(mydir1, mydir2, hlim=c(19,21), vlim=c(9,12))

# same as last, but only use data from 2005-2010
mfBURNEDAREA(mydir1, mydir2, hlim=c(19,21), vlim=c(9,12), ylim=c(2005,2010))

# same as last, without download.
# images were acquired otherwise.
mfBURNEDAREA(mydir1, mydir2, download=FALSE, hlim=c(19,21), vlim=c(9,12), ylim=c(2005,2010))

}
}

